# SPDX-FileCopyrightText: 2021-2023 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later

"""
Pose Library - GUI definition.
"""

import bpy
from . import common

# Main Panel
class DATA_PT_DSPLPanel(bpy.types.Panel):
    bl_label = "Damn Simple Pose Library"
    bl_id = "DATA_PT_DSPLPanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Pose'

    @classmethod
    def poll(cls, context):
        return len(bpy.context.selected_objects)

    def draw(self, context):
        dspl_panel_layout = self.layout

        # Detect Armature object and parent
        armature_layout = dspl_panel_layout.column(align=True)
        active_obj = context.active_object
        arm_object = common.getArmatureObject(context)

        if arm_object:
            if arm_object == active_obj:
                armature_layout.label(text=arm_object.name + " (Active)")
            elif arm_object == active_obj.parent:
                armature_layout.label(text=arm_object.name + " (Parent)")

            # Show or create pose library
            pose_library_dspl = common.getDsplAction(context)
            pose_library_action = common.getArmatureAction(context)
            
            if pose_library_dspl or pose_library_action:
                if pose_library_dspl and not pose_library_action:
                    active_pose_library = pose_library_dspl
                    armature_layout.template_ID(arm_object.dspl, "pose_library", new="dspl.create_pose_library")

                elif pose_library_action and not pose_library_dspl:
                    active_pose_library = pose_library_action
                    armature_layout.template_ID(arm_object.animation_data, "action", new="dspl.create_pose_library")
                    armature_layout.label(text="Legacy Pose Library detected.")
                    armature_layout.label(text="Click to mount as Pose Library")
                    armature_layout.operator("dspl.convert_pose_library", icon='PLUGIN', text="Convert to DSPL")
                
                elif pose_library_action and pose_library_dspl:
                    armature_layout.template_ID(arm_object.dspl, "pose_library", new="dspl.create_pose_library")
                    armature_layout.label(text="Pose library is active action")
                    armature_layout.operator("dspl.convert_pose_library", icon='PLUGIN', text="Convert to DSPL")
                    active_pose_library = pose_library_dspl

                if active_pose_library:
                    # Quick controls
                    quick_pose_controls_layout = dspl_panel_layout.column()
                    quick_pose_controls_layout.menu(OBJECT_MT_AddPoseMenu.bl_idname, icon='ADD', text="New Pose")
                    quick_apply_layout = quick_pose_controls_layout.split(align=True)
                    quick_apply_layout.prop(arm_object.dsplvars, "only_selected", icon='GROUP_BONE', text="", toggle=True)
                    quick_apply_layout.operator("dspl.apply_pose", icon='ARMATURE_DATA', text="Apply Pose")

                    # List poses in pose library
                    pose_box_layout = dspl_panel_layout.row()
                    pose_list_layout = pose_box_layout.column()
                    pose_list_layout.template_list("UI_UL_list", "pose_markers", active_pose_library, "pose_markers",
                        active_pose_library.pose_markers, "active_index", rows=4)

                    # Pose operators
                    pose_ops_layout = pose_box_layout.column(align=True)

                    # Add Pose
                    pose_ops_layout.operator("dspl.draw_new_pose_menu", icon='ADD', text="")

                    pose_marker_active = active_pose_library.pose_markers.active
                    if pose_marker_active is not None:
                        # Remove pose
                        pose_ops_layout.operator("dspl.remove_pose", icon='REMOVE', text="")

                        # Apply pose
                        pose_ops_layout.operator("dspl.apply_pose", icon='ARMATURE_DATA', text="")

                        # Reorder poses
                        pose_ops_layout.operator("dspl.move_pose", icon='TRIA_UP', text="").direction = "UP"
                        pose_ops_layout.operator("dspl.move_pose", icon='TRIA_DOWN', text="").direction = "DOWN"
                
                

            else:
                armature_layout.label(text="No Action or Pose Library detected")
                armature_layout.template_ID(arm_object.dspl, "pose_library", new="dspl.create_pose_library")

        else:
            armature_layout.label(text="No armature or parent selected")

# Add Menu Popup
class OBJECT_MT_AddPoseMenu(bpy.types.Menu):
    bl_idname = "OBJECT_MT_AddPoseMenu"
    bl_label = "Add Pose"

    def draw(self, context):
        dspl_add_menu_layout = self.layout

        dspl_add_menu_layout.operator("dspl.draw_new_pose_menu", icon='DECORATE_DRIVER', text="Add New Pose")
        dspl_add_menu_layout.operator("dspl.replace_pose", icon='DECORATE_OVERRIDE', text="Replace Existing Pose")

classes = (
    DATA_PT_DSPLPanel,
    OBJECT_MT_AddPoseMenu
)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

def unregister():
    from bpy.utils import unregister_class
    for cls in classes:
        unregister_class(cls)
