import bpy
import mathutils
from . import common
from . import gui

# Operator to create a new pose library
class DSPL_OT_CreatePoseLibrary(bpy.types.Operator):
    bl_idname = "dspl.create_pose_library"
    bl_label = "Create Pose Library"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Create new Pose Library"

    def execute(self, context):
        arm_object = common.getArmatureObject(context)
        arm_object.dspl.pose_library = bpy.data.actions.new(name=arm_object.name + "_PoseLib")

        return {'FINISHED'}

# Operator to convert a new pose library
class DSPL_OT_ConvertPoseLibrary(bpy.types.Operator):
    bl_idname = "dspl.convert_pose_library"
    bl_label = "Convert Pose Library"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Convert Pose Library"

    def execute(self, context):
        arm_object = common.getArmatureObject(context)
        arm_object.dspl.pose_library = arm_object.animation_data.action
        arm_object.animation_data.action = None

        return {'FINISHED'}

# Operator to to draw a menu for new poses
class DSPL_OT_DrawNewPoseMenu(bpy.types.Operator):
    bl_idname = "dspl.draw_new_pose_menu"
    bl_label = "New Pose"
    bl_description = "Open menu"
    bl_options = {'INTERNAL'}

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self, width=200)

    def check(self, context):
        return True

    def draw(self, context):
        dspl_create_popup_layout = self.layout
        dspl_new_pose_menu = dspl_create_popup_layout.box()

        arm_object = common.getArmatureObject(context)
        pose_library = common.getPoseLib(context)
        dspl_new_pose_menu.prop(arm_object.dsplvars, "pose_new_name", text="Name")
        dspl_new_pose_menu.prop(arm_object.dsplvars, "only_selected", toggle=True)
        dspl_new_pose_menu.operator("dspl.add_pose", icon='ADD', text="Add New Pose").posename = arm_object.dsplvars.pose_new_name

# Operator to to call up popup menus by ID
class DSPL_OT_CallPopupMenu(bpy.types.Operator):
    bl_idname = "dspl.call_popup_menu"
    bl_label = "Popup menu"
    bl_description = "Open menu"
    bl_options = {'REGISTER', 'UNDO'}

    menuID: bpy.props.StringProperty()

    def execute(self, context):
        bpy.ops.wm.call_menu(name=self.menuID)

        return {'FINISHED'}

# Operator to add keyframes and marker to pose library
class DSPL_OT_AddPose(bpy.types.Operator):
    bl_idname = "dspl.add_pose"
    bl_label = "Add Pose"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Add new pose"

    posename: bpy.props.StringProperty()

    def findFcurve(self, context, bone_name, transform, index_int):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_frame = pose_markers.active.frame
        
        fcurve_object = action_object.fcurves.find('pose.bones["'+bone_name+'"].'+transform+'', index=index_int)
        if hasattr(fcurve_object, 'evaluate'):
            return fcurve_object.evaluate(active_frame)
        else:
            return 0.0

    def createFcurve(self, context, bone_name, transform, index_int):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        try:
            action_object.fcurves.new('pose.bones["'+bone_name+'"].'+transform+'', index=index_int, action_group=bone_name)
            return
        except:
            pass

    def createKeyframes(self, context, bone_name, transform, index_int, new_marker, loc):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        try:
            action_object.fcurves.find('pose.bones["'+bone_name+'"].'+transform+'', index=index_int).keyframe_points.insert(new_marker, loc)
            return
        except:
            pass

    def execute(self, context):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        new_name = self.posename
        counter = 1
        
        for i in range(bpy.context.scene.frame_start, bpy.context.scene.frame_end):
            for pm in pose_markers:
                name_check = pm.name
                frame_check = pm.frame
                if frame_check == i:
                    break
            else:
                new_marker = i
                break


        while pose_markers.find(new_name) > -1:
            print("Duplicate posename detected")
            new_name = self.posename + ".{:03d}".format(counter)
            counter += 1
        else:
            pose_name = new_name

        pose_markers.new(name=pose_name)
        pose_markers[pose_name].frame = new_marker

        for bone in arm_object.pose.bones:
            if bone.bone.select or arm_object.dsplvars.only_selected == False:
                bone_name = bone.name

                if bone.rotation_mode == "XYZ":
                    rot_mode = "rotation_euler"
                elif bone.rotation_mode == "QUATERNION":
                    rot_mode = "rotation_quaternion"

                loc_x = bone.location[0]
                loc_y = bone.location[1]
                loc_z = bone.location[2]
                self.createFcurve(context, bone_name, "location", 0)
                self.createFcurve(context, bone_name, "location", 1)
                self.createFcurve(context, bone_name, "location", 2)
                self.createKeyframes(context, bone_name, "location", 0, new_marker, loc_x)
                self.createKeyframes(context, bone_name, "location", 1, new_marker, loc_y)
                self.createKeyframes(context, bone_name, "location", 2, new_marker, loc_z)
                if rot_mode == "rotation_quaternion":
                    rot_w = bone.rotation_quaternion[0]
                    rot_x = bone.rotation_quaternion[1]
                    rot_y = bone.rotation_quaternion[2]
                    rot_z = bone.rotation_quaternion[3]
                    self.createFcurve(context, bone_name, rot_mode, 0)
                    self.createFcurve(context, bone_name, rot_mode, 1)
                    self.createFcurve(context, bone_name, rot_mode, 2)
                    self.createFcurve(context, bone_name, rot_mode, 3)
                    self.createKeyframes(context, bone_name, rot_mode, 0, new_marker, rot_w)
                    self.createKeyframes(context, bone_name, rot_mode, 1, new_marker, rot_x)
                    self.createKeyframes(context, bone_name, rot_mode, 2, new_marker, rot_y)
                    self.createKeyframes(context, bone_name, rot_mode, 3, new_marker, rot_z)
                elif rot_mode == "rotation_euler":
                    rot_x = bone.rotation_euler[0]
                    rot_y = bone.rotation_euler[1]
                    rot_z = bone.rotation_euler[2]
                    self.createFcurve(context, bone_name, rot_mode, 0)
                    self.createFcurve(context, bone_name, rot_mode, 1)
                    self.createFcurve(context, bone_name, rot_mode, 2)
                    self.createKeyframes(context, bone_name, rot_mode, 0, new_marker, rot_x)
                    self.createKeyframes(context, bone_name, rot_mode, 1, new_marker, rot_y)
                    self.createKeyframes(context, bone_name, rot_mode, 2, new_marker, rot_z)
                scl_x = bone.scale[0]
                scl_y = bone.scale[1]
                scl_z = bone.scale[2]
                self.createFcurve(context, bone_name, "scale", 0)
                self.createFcurve(context, bone_name, "scale", 1)
                self.createFcurve(context, bone_name, "scale", 2)
                self.createKeyframes(context, bone_name, "scale", 0, new_marker, scl_x)
                self.createKeyframes(context, bone_name, "scale", 1, new_marker, scl_y)
                self.createKeyframes(context, bone_name, "scale", 2, new_marker, scl_z)
        action_object.pose_markers.active = pose_markers[pose_name]
        bpy.context.area.tag_redraw()

        return {'FINISHED'}

# Operator to replace the current pose
class DSPL_OT_ReplacePose(bpy.types.Operator):
    bl_idname = "dspl.replace_pose"
    bl_label = "Replace Pose"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Replace existing pose"

    def createFcurve(self, context, bone_name, transform, index_int):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        try:
            action_object.fcurves.new('pose.bones["'+bone_name+'"].'+transform+'', index=index_int, action_group=bone_name)
            return
        except:
            pass

    def createKeyframes(self, context, bone_name, transform, index_int, new_marker, loc):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        try:
            action_object.fcurves.find('pose.bones["'+bone_name+'"].'+transform+'', index=index_int).keyframe_points.insert(new_marker, loc)
            return
        except:
            pass

    def execute(self, context):

        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_marker = pose_markers.active
        new_name = active_marker.name
        counter = 1

        for pm in pose_markers:
            name_check = pm.name
            frame_check = pm.frame
            if name_check == new_name:
                target_name = name_check
                target_frame = frame_check
                break
        else:
            return

        new_marker = target_frame

        for bone in arm_object.pose.bones:
            if bone.bone.select or arm_object.dsplvars.only_selected == False:
                bone_name = bone.name

                if bone.rotation_mode == "XYZ":
                    rot_mode = "rotation_euler"
                elif bone.rotation_mode == "QUATERNION":
                    rot_mode = "rotation_quaternion"

                loc_x = bone.location[0]
                loc_y = bone.location[1]
                loc_z = bone.location[2]
                self.createFcurve(context, bone_name, "location", 0)
                self.createFcurve(context, bone_name, "location", 1)
                self.createFcurve(context, bone_name, "location", 2)
                self.createKeyframes(context, bone_name, "location", 0, new_marker, loc_x)
                self.createKeyframes(context, bone_name, "location", 1, new_marker, loc_y)
                self.createKeyframes(context, bone_name, "location", 2, new_marker, loc_z)
                if rot_mode == "rotation_quaternion":
                    rot_w = bone.rotation_quaternion[0]
                    rot_x = bone.rotation_quaternion[1]
                    rot_y = bone.rotation_quaternion[2]
                    rot_z = bone.rotation_quaternion[3]
                    self.createFcurve(context, bone_name, rot_mode, 0)
                    self.createFcurve(context, bone_name, rot_mode, 1)
                    self.createFcurve(context, bone_name, rot_mode, 2)
                    self.createFcurve(context, bone_name, rot_mode, 3)
                    self.createKeyframes(context, bone_name, rot_mode, 0, new_marker, rot_w)
                    self.createKeyframes(context, bone_name, rot_mode, 1, new_marker, rot_x)
                    self.createKeyframes(context, bone_name, rot_mode, 2, new_marker, rot_y)
                    self.createKeyframes(context, bone_name, rot_mode, 3, new_marker, rot_z)
                elif rot_mode == "rotation_euler":
                    rot_x = bone.rotation_euler[0]
                    rot_y = bone.rotation_euler[1]
                    rot_z = bone.rotation_euler[2]
                    self.createFcurve(context, bone_name, rot_mode, 0)
                    self.createFcurve(context, bone_name, rot_mode, 1)
                    self.createFcurve(context, bone_name, rot_mode, 2)
                    self.createKeyframes(context, bone_name, rot_mode, 0, new_marker, rot_x)
                    self.createKeyframes(context, bone_name, rot_mode, 1, new_marker, rot_y)
                    self.createKeyframes(context, bone_name, rot_mode, 2, new_marker, rot_z)
                scl_x = bone.scale[0]
                scl_y = bone.scale[1]
                scl_z = bone.scale[2]
                self.createFcurve(context, bone_name, "scale", 0)
                self.createFcurve(context, bone_name, "scale", 1)
                self.createFcurve(context, bone_name, "scale", 2)
                self.createKeyframes(context, bone_name, "scale", 0, new_marker, scl_x)
                self.createKeyframes(context, bone_name, "scale", 1, new_marker, scl_y)
                self.createKeyframes(context, bone_name, "scale", 2, new_marker, scl_z)

        return {'FINISHED'}

# Operator to remove keyframes and marker
class DSPL_OT_RemovePose(bpy.types.Operator):
    bl_idname = "dspl.remove_pose"
    bl_label = "Remove Pose"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Remove pose"

    def execute(self, context):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_index = pose_markers.active_index
        active_marker = pose_markers.active
        active_frame = active_marker.frame

        fcurves = action_object.fcurves
        for fcu in fcurves:
            for kf in fcu.keyframe_points.values():
                if kf.co.x == active_frame:
                    fcu.keyframe_points.remove(kf)

        if active_index <= 0:
            next_index = active_index
            next_marker = pose_markers[next_index]
        elif (active_index + 1) >= len(pose_markers):
            next_index = active_index - 1
            next_marker = pose_markers[next_index]
        else:
            next_index = active_index
            next_marker = pose_markers[next_index]

        pose_markers.remove(marker=active_marker)
        action_object.pose_markers.active = next_marker
        action_object.pose_markers.active_index = next_index

        return {'FINISHED'}

# Operator to apply a pose from active marker
class DSPL_OT_ApplyPose(bpy.types.Operator):
    bl_idname = "dspl.apply_pose"
    bl_label = "Apply Pose"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Apply pose"

    def findFcurve(self, context, bone_name, transform, index_int):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_frame = pose_markers.active.frame
        
        fcurve_object = action_object.fcurves.find('pose.bones["'+bone_name+'"].'+transform+'', index=index_int)
        if hasattr(fcurve_object, 'evaluate'):
            return fcurve_object.evaluate(active_frame)
        else:
            return 0.0

    def execute(self, context):
        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_marker = pose_markers.active
        active_index = pose_markers.active_index
        active_frame = active_marker.frame
        active_posename = active_marker.name

        for bone in arm_object.pose.bones:
            if bone.bone.select or arm_object.dsplvars.only_selected == False:
                bone_name = bone.name
        
                if bone.rotation_mode == "XYZ":
                    rot_mode = "rotation_euler"
                elif bone.rotation_mode == "YZX":
                    rot_mode = "rotation_euler"
                elif bone.rotation_mode == "ZXY":
                    rot_mode = "rotation_euler"
                elif bone.rotation_mode == "QUATERNION":
                    rot_mode = "rotation_quaternion"
                else:
                    print("Unsupported bone!")
                    rot_mode = None

                loc_x = self.findFcurve(context, bone_name, "location", 0)
                loc_y = self.findFcurve(context, bone_name, "location", 1)
                loc_z = self.findFcurve(context, bone_name, "location", 2)
                if rot_mode == "rotation_quaternion":
                    rot_w = self.findFcurve(context, bone_name, rot_mode, 0)
                    rot_x = self.findFcurve(context, bone_name, rot_mode, 1)
                    rot_y = self.findFcurve(context, bone_name, rot_mode, 2)
                    rot_z = self.findFcurve(context, bone_name, rot_mode, 3)
                elif rot_mode == "rotation_euler":
                    rot_x = self.findFcurve(context, bone_name, rot_mode, 0)
                    rot_y = self.findFcurve(context, bone_name, rot_mode, 1)
                    rot_z = self.findFcurve(context, bone_name, rot_mode, 2)
                scl_x = self.findFcurve(context, bone_name, "scale", 0)
                scl_y = self.findFcurve(context, bone_name, "scale", 1)
                scl_z = self.findFcurve(context, bone_name, "scale", 2)

                bone.location = mathutils.Vector((loc_x, loc_y, loc_z))
                if bone.rotation_mode == "XYZ":
                    bone.rotation_euler = mathutils.Euler((rot_x, rot_y, rot_z))
                elif bone.rotation_mode == "YZX":
                    bone.rotation_euler = mathutils.Euler((rot_y, rot_z, rot_x))
                elif bone.rotation_mode == "ZXY":
                    bone.rotation_euler = mathutils.Euler((rot_z, rot_x, rot_y))
                elif bone.rotation_mode == "YXZ":
                    bone.rotation_euler = mathutils.Euler((rot_y, rot_x, rot_z))
                elif bone.rotation_mode == "XZY":
                    bone.rotation_euler = mathutils.Euler((rot_x, rot_z, rot_y))
                elif rot_mode == "rotation_quaternion":
                    bone.rotation_quaternion = mathutils.Quaternion((rot_w, rot_x, rot_y, rot_z))
                bone.scale = mathutils.Vector((scl_x, scl_y, scl_z))

        return {'FINISHED'}

# Operator to reorder pose markers
class DSPL_OT_MovePose(bpy.types.Operator):
    bl_idname = "dspl.move_pose"
    bl_label = "Move Pose"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "Move pose"

    direction: bpy.props.StringProperty(name="Direction", default="DOWN")

    def execute(self, context):

        arm_object = common.getArmatureObject(context)
        action_object = common.getPoseLib(context)
        pose_markers = action_object.pose_markers
        active_marker = pose_markers.active
        active_index = pose_markers.active_index

        if self.direction == "UP":
            move_dir = -1
        elif self.direction == "DOWN":
            move_dir = 1

        swap_index = active_index + move_dir
        swap_marker = pose_markers[swap_index]

        tmp_marker_name = swap_marker.name
        tmp_marker_frame = swap_marker.frame
        action_object.pose_markers[swap_index].name = active_marker.name
        action_object.pose_markers[swap_index].frame = active_marker.frame
        action_object.pose_markers[active_index].name = tmp_marker_name
        action_object.pose_markers[active_index].frame = tmp_marker_frame
        action_object.pose_markers.active_index = swap_index

        return {'FINISHED'}

classes = (
   DSPL_OT_CreatePoseLibrary,
   DSPL_OT_ConvertPoseLibrary,
   DSPL_OT_CallPopupMenu,
   DSPL_OT_DrawNewPoseMenu,
   DSPL_OT_AddPose,
   DSPL_OT_ReplacePose,
   DSPL_OT_RemovePose,
   DSPL_OT_ApplyPose,
   DSPL_OT_MovePose
)

register, unregister = bpy.utils.register_classes_factory(classes)
